/*
 * AbstractLinkedNode.java
 * Tomokazu ARITA
 * arita@obirin.ac.jp
 * Created on 2003/08/11, 3:07
 */

package org.hichart.h3code;

/**
 * AbstractLinkedNode (node for rectangular dissection graph)
 * @version 0.1.0 2003/08/11
 * @author Tomokazu ARITA (arita@obirin.ac.jp)
 *
 */


public abstract class AbstractLinkedNode {
    
    /* Position */
	
    /**
     * <code>NWE_LEFT</code> is the "North Wall Edge (LEFT)" position name in 8 pointers. 
     */
    public static final int NWE_LEFT  = 1;
    
    /**
     * <code>NWE_RIGHT</code> is the "North Wall Edge (RIGHT)" position name in 8 pointers. 
     */
    public static final int NWE_RIGHT = 2;
    
    /**
     * <code>SWE_LEFT</code> is the "South Wall Edge (LEFT)" position name in 8 pointers. 
     */
    public static final int SWE_LEFT  = 3;
    
    /**
     * <code>SWE_RIGHT</code> is the "South Wall Edge (RIGHT)" position name in 8 pointers. 
     */
    public static final int SWE_RIGHT = 4;
    
    /**
     * <code>WWE_UPPER</code> is the "West Wall Edge (UPPER)" position name in 8 pointers. 
     */
    public static final int WWE_UPPER = 5;
    
    /**
     * <code>WWE_LOWER</code> is the "West Wall Edge (LOWER)" position name in 8 pointers. 
     */
    public static final int WWE_LOWER = 6;
    
    /**
     * <code>EWE_UPPER</code> is the "East Wall Edge (UPPER)" position name in 8 pointers. 
     */
    public static final int EWE_UPPER = 7;
    
    /**
     * <code>EWE_LOWER</code> is the "East Wall Edge (LOWER)" position name in 8 pointers. 
     */
    public static final int EWE_LOWER = 8;
    
    
    /* Fields */
    
    /**
     * <code>nweLeft</code> is a field for the "North Wall Edge (LEFT)" pointer of a node instance.
     * The value of <code>nweLeft</code> is an ID number of adjacent node that connects this node instance of a north wall edge (left).  
     * The value "0" denotes null pointer. 
     */
    private int nweLeft   = 0;
    
    /**
     * <code>nweRight</code> is a field for the "North Wall Edge (RIGHT)" pointer of a node instance.
     * The value of <code>nweRight</code> is an ID number of adjacent node that connects this node instance of a north wall edge (right). 
     * The value "0" denotes null pointer. 
     */
    private int nweRight  = 0;
    
    /**
     * <code>sweLeft</code> is a field for the "South Wall Edge (LEFT)" pointer of a node instance.
     * The value of <code>sweLeft</code> is an ID number of adjacent node that connects this node instance of a south wall edge (left).
     * The value "0" denotes null pointer. 
     */
    private int sweLeft   = 0;
    
    /**
     * <code>sweRight</code> is a field for the "South Wall Edge (RIGHT)" pointer of a node instance.
     * The value of <code>sweRight</code> is an ID number of adjacent node that connects this node instance of a south wall edge (right).
     * The value "0" denotes null pointer. 
     */
    private int sweRight  = 0;
    
    /**
     * <code>wweUpper</code> is a field for the "West Wall Edge (Upper)" pointer of a node instance.
     * The value of <code>wweUpper</code> is an ID number of adjacent node that connects this node instance of a west wall edge (upper).
     * The value "0" denotes null pointer. 
     */
    private int wweUpper  = 0;
    
    /**
     * <code>wweLower</code> is a field for the "West Wall Edge (LOWER)" pointer of a node instance.
     * The value of <code>wweLower</code> is an ID number of adjacent node that connects this node instance of a west wall edge (lower).
     * The value "0" denotes null pointer. 
     */
    private int wweLower  = 0;
    
    /**
     * <code>eweUpper</code> is a field for the "East Wall Edge (UPPER)" pointer of a node instance.
     * The value of <code>eweUpper</code> is an ID number of adjacent node that connects this node instance of an east wall edge (upper).
     * The value "0" denotes null pointer. 
     */
    private int eweUpper  = 0;
    
    /**
     * <code>eweLower</code> is a field for the "East Wall Edge (LOWER)" pointer of a node instance.
     * The value of <code>eweLower</code> is an ID number of adjacent node that connects this node instance of an east wall edge (lower).
     * The value "0" denotes null pointer. 
     */
    private int eweLower  = 0;
    
    
    
    /**
     * <code>id</code> denotes the node ID. 
     * Node ID needs to create an instance. 
     */
    private int id = 0;
    
    /* NEW FIELD 041213 */
    //public boolean target =false; 
    
    /** Creates a new instance of AbstractLinkedNode */
    public AbstractLinkedNode(int id) {
        this.id = id;
    }
    
    /* connect \bh */
    /* ̃\bh́C̃CX^X̊eӃtB[h
     liׂ̒_ԍjZbg邾̊֐łD
     ̃\bh́C̃NȂD
     ̐ڑɂ́CGraphNX
     edgeConstruct()\bh𗘗pȂĂ͂ȂȂD
     CӂŐڑ\ȂtrueԂCڑłȂfalseԂD
     gp@Fconnect(node.getID(), AbstractLinkedNode.NWE_LEFT);
   */
    /**
     * Connects this instance to an adjacent node.  
     * @param node_number  an adjacent node ID
     * @param position     <code>NWE_LEFT</code>, <code>NWE_RIGHT</code>, <code>SWE_LEFT</code>, <code>SWE_RIGHT</code>, <code>WWE_UPPER</code>, <code>WWE_LOWER</code>, <code>EWE_UPPER</code>, or <code>EWE_LOWER</code>
     * @return  if this instance was connected to an adjacent node, then this method returns <code>true</code>. 
     */
    public boolean connect(int node_number, int position){
        switch(position){
            case NWE_LEFT: 
                if(nweLeft == 0){ 
                    nweLeft = node_number; 
                    return true; 
                }else{
                    return false;
                }
            case NWE_RIGHT:
                 if(nweRight == 0){ 
                    nweRight = node_number; 
                    return true; 
                }else{
                    return false;
                }
            case SWE_LEFT: 
                 if(sweLeft == 0){ 
                    sweLeft = node_number; 
                    return true; 
                }else{
                    return false;
                }
            case SWE_RIGHT:
                 if(sweRight == 0){ 
                    sweRight = node_number; 
                    return true; 
                }else{
                    return false;
                }
            case WWE_UPPER:
                 if(wweUpper == 0){ 
                    wweUpper = node_number; 
                    return true; 
                }else{
                    return false;
                }
            case WWE_LOWER:
                if(wweLower == 0){ 
                    wweLower = node_number; 
                    return true; 
                }else{
                    return false;
                }
            case EWE_UPPER:
                if(eweUpper == 0){ 
                    eweUpper = node_number; 
                    return true; 
                }else{
                    return false;
                }
            case EWE_LOWER:
                 if(eweLower == 0){ 
                    eweLower = node_number; 
                    return true; 
                }else{
                    return false;
                }
            default: 
                return false;
        }
    }
    
    /* discconect\bh */
    /* ӂؒfD
     positionɕӂΐؒftrueԂD
     ӂȂ΁CfalseԂD
   */
    /**
     * Disconnects an adjacent node from this instance. 
     * @param position   <code>NWE_LEFT</code>, <code>NWE_RIGHT</code>, <code>SWE_LEFT</code>, <code>SWE_RIGHT</code>, <code>WWE_UPPER</code>, <code>WWE_LOWER</code>, <code>EWE_UPPER</code>, or <code>EWE_LOWER</code>
     * @return  if an adjacent node was disconnect, then this method returns <code>true</code>. 
     */
    public boolean disconnect(int position){
        switch(position){
            case NWE_LEFT: 
                if(nweLeft != 0){ 
                    nweLeft = 0; 
                    return true; 
                }else{
                    return false;
                }
            case NWE_RIGHT:
                 if(nweRight != 0){ 
                    nweRight = 0; 
                    return true; 
                }else{
                    return false;
                }
            case SWE_LEFT: 
                 if(sweLeft != 0){ 
                    sweLeft = 0; 
                    return true; 
                }else{
                    return false;
                }
            case SWE_RIGHT:
                 if(sweRight != 0){ 
                    sweRight = 0; 
                    return true; 
                }else{
                    return false;
                }
            case WWE_UPPER:
                 if(wweUpper != 0){ 
                    wweUpper = 0; 
                    return true; 
                }else{
                    return false;
                }
            case WWE_LOWER:
                if(wweLower != 0){ 
                    wweLower = 0; 
                    return true; 
                }else{
                    return false;
                }
            case EWE_UPPER:
                if(eweUpper != 0){ 
                    eweUpper = 0; 
                    return true; 
                }else{
                    return false;
                }
            case EWE_LOWER:
                 if(eweLower != 0){ 
                    eweLower = 0; 
                    return true; 
                }else{
                    return false;
                }
            default: 
                return false;
        }
    }
    
    
    /**
     * Sets a node ID to this instance. 
     * @param id   node ID
     */
    public void setID(int id){
        this.id = id;
    }
    
    
    /**
     * Gets node ID of this instance. 
     * @return   node ID
     */
    public int getID(){
        return id;
    }
    
    /* 폜֐ destroy() */
    /* CX^Xł鎞ɕKĂяoȂ΂ȂȂ */
    /**
     * Destroys this instance. 
     * @return true or false
     */
    public abstract boolean destroy();  
    
    /* אڃZIDԂ */
    /* qĂȂCpositionsȂ0Ԃ */
    /**
     * Gets an adjacent node ID. 
     * @param position   <code>NWE_LEFT</code>, <code>NWE_RIGHT</code>, <code>SWE_LEFT</code>, <code>SWE_RIGHT</code>, <code>WWE_UPPER</code>, <code>WWE_LOWER</code>, <code>EWE_UPPER</code>, or <code>EWE_LOWER</code>
     * @return  an adjacent node ID. If a return value is 0, then there is no adjacent node. 
     */
    public int getConnectedCell(int position){
        switch(position){
            case NWE_LEFT: 
                return nweLeft; 
            case NWE_RIGHT:
                return nweRight;
            case SWE_LEFT: 
                return sweLeft;
            case SWE_RIGHT:
                return sweRight;
            case WWE_UPPER:
                return wweUpper;
            case WWE_LOWER:
                return wweLower;
            case EWE_UPPER:
                return eweUpper;
            case EWE_LOWER:
                return eweLower;
            default: 
                return 0;
        }
    }
}
